"""
Iacli library
~~~~~~~~~~~~~

Iacli is the CLI driver for the internetarchive python library:

<https://github.com/jjjake/ia-wrapper>.

:copyright: (c) 2013 by Jacob M. Johnson.
:license: GPL, see LICENSE for more details.

"""
from internetarchive import __version__, __author__, __license__, __copyright__



__title__ = 'iacli'
__version__ = __version__
__author__ = __author__
__license__ = __license__
__copyright__ = __copyright__
